import {Post, Tag, LogEntry, LogType} from "../type/generic";
import {Scrapper}                     from "../class/Scrapper";
import {getPageContents}              from "../helper/requestManager";
import * as cheerio from 'cheerio';

export class Rule34xxx extends Scrapper {

    constructor() {
        // Set the domain base of the current Scrapper as "rule34.xxx"
        super("https://rule34.xxx");
    }

    /**
     * Get the details of a specific post
     * @param url The URL to the post, this must be the actual page which contains the image, tags, etc...
     */
    public async getPostDetails( url: string ): Promise<Post | null> {

        // Check if the provided link is valid
        if ( !this.checkURLBase(url) ) {
            throw new Error(`Invalid url provided`);
        }

        // Send out the request to grab the contents of the post


        return null;
    }

    /**
     * Get a list of posts from the mentioned page
     * @param url 
     * @returns 
     */
    public async getPostsFromPage( url: string ): Promise<Array<string>> {

        // Check if the provided link is valid
        if ( !this.checkURLBase(url) ) {
            throw new Error(`Invalid url provided`);
        }

        // Initialize the page contents here
        let pageContents: string = null;

        // Send out the request to grab the contents of the post
        try {
            // Send out the initial Axios request to fetch the data from the page
            await getPageContents(url)
            .then( request => {
                if ( request.status < 200 || request.status > 299 ) {
                    this.logs.push({
                        msg: `Invalid response code[${request.status}]`, 
                        type: LogType.ERROR,
                        err:  null,
                        data: null,
                        ts: new Date()
                    });
                    throw new Error(`Invalid response code[${request.status}]`);
                }
        
                pageContents = (request.data as string);
            })
    
    
        } catch ( err ) {
            // "Handle" the error so that it's in the above .catch
            this.logs.push({
                msg: `[Error]::getPostsFromPage::`, 
                type: LogType.ERROR,
                err: (err as Error),
                data: null,
                ts: new Date()
            });
            throw err;
        }

        // Process the page's posts with cheerio
        const $ = cheerio.load((pageContents as string));

        // Define the post List
        const postList: Array<string> = [];

        // Workaround I guess
        let self = this;

        // Go through all of the posts
        $(`.thumb`).each( function() {
            const href = $(this).find(`a`).attr(`href`);
            if ( `${href}`.length >= `index.php?page=post&s=view&id=`.length )
                postList.push(`${self.domain}/${href}`);
        });

        return postList;
    }

    public async crawlPages( url: string, pageCount: number = 10, batchSize: number = 1 ): Promise<Array<string>> {

        // Check if the provided link is valid
        if ( !this.checkURLBase(url) ) {
            throw new Error(`Invalid url provided`);
        }

        // A list of all of the found pages
        let foundPages = new Array<string>();
        // The next url we are hitting
        let nextPage: string = url;

        // Go through as many pages as requested
        for ( let i = 0; i < pageCount; i++ ) {

            if ( this.verbose ) {
                console.log(`[${i+1}/${pageCount}]Crawling ${nextPage}`);
            }

            // Initialize the page contents here
            let pageContents: string = null;

            // Try and find the button to the next page
            try {
                // Send out the initial Axios request to fetch the data from the page
                await getPageContents(nextPage)
                .then( request => {
                    if ( request.status < 200 || request.status > 299 ) {
                        this.logs.push({
                            msg: `Invalid response code[${request.status}]`, 
                            type: LogType.ERROR,
                            err:  null,
                            data: null,
                            ts: new Date()
                        });
                        throw new Error(`Invalid response code[${request.status}]`);
                    }
            
                    pageContents = (request.data as string);
                })
        
        
            } catch ( err ) {
                // "Handle" the error so that it's in the above .catch
                this.logs.push({
                    msg: `[Error]::getPostsFromPage::`, 
                    type: LogType.ERROR,
                    err: (err as Error),
                    data: null,
                    ts: new Date()
                });
                throw err;
            }

            // Process the page's posts with cheerio
            const $ = cheerio.load((pageContents as string));

            // Add the current page we are on to the list
            foundPages.push(nextPage);

            const nextPageButton = $(`a[alt="next"]`);
            if ( nextPageButton.length > 0 ) {
                nextPage = `${this.domain}/` + nextPageButton.attr(`href`);
            } else {
                // Since we didn't find the proper button, skip this page.
                break;
            }
        }

        // Return the found pages
        return foundPages;
    }

}